@@ -1,4 +1,5 @@
 import datetime
+import logging
 from npd_cat_corr.client import BloombergClient
 import pandas as pd
 import os
@@ -11,31 +12,53 @@ NON_SALES_COLS = ['Ticker','quarter_end','date', 'eps_estimate',
 def get_quarter_end(date, calendar_type):
 
     fiscal_months = [calendar_type + x for x in [0, 3, 6, 9]]
+    if date.month - 1 in fiscal_months and date.day < 15:
+        last_day = calendar.monthrange(date.year, date.month-1)[1]
+        date = datetime.datetime(date.year, date.month-1, last_day)
+    elif date.month == 1 and 12 in fiscal_months and date.day < 15:
+        last_day = calendar.monthrange(date.year-1, 12)[1]
+        date = datetime.datetime(date.year-1, 12, last_day)
+
     for month in fiscal_months:
         if date.month <= month:
             last_day = calendar.monthrange(date.year, month)[1]
             return datetime.datetime(date.year, month, last_day)
-    last_day = calendar.monthrange(date.year + 1, fiscal_months[0])[1]
-    return datetime.datetime(date.year + 1, fiscal_months[0], last_day)
+    last_day = calendar.monthrange(date.year+1, fiscal_months[0])[1]
+    return datetime.datetime(date.year+1, fiscal_months[0], last_day)
 
-def create_dataset(ew_data):
+def choose_calendar_type(dates):
+
+    ctypes = {1:0, 2:0, 3:0}
+    for d in dates:
+        last_day = calendar.monthrange(d.year, d.month)[1]
+        if d.day / last_day >= 0.5:
+            this_month = d.month
+        elif d.month > 1:
+            this_month = d.month - 1
+        else:
+            this_month = 12
+        if this_month in [1,4,7,10]:
+            ctypes[1] += 1
+        elif this_month in [2,5,8,11]:
+            ctypes[2] += 1
+        elif this_month in [3,6,9,12]:
+            ctypes[3] += 1
+    return max(ctypes, key = lambda k: ctypes[k])
+
+def create_dataset(ew_data, log=False):
 
     ew_data = ew_data[(~ew_data["Ticker"].str.contains(" ", regex=False)) & (~ew_data["Ticker"].str.contains(".", regex=False))]
     tickers = list(ew_data["Ticker"].unique())
     bclient = BloombergClient()
-    bdata = bclient.get_quarterly_financials(tickers)
+    bdata = bclient.get_quarterly_financials(tickers, log=log)
     bdata["q_datetime"] = pd.to_datetime(bdata["date"])
     
+    if log:
+        logging.info("Inferring Company Financial Calendars")
     #construct financial calendar map
     ticker_quarters = {}
     for ticker in bdata["symbol"].unique():
-        qdate = bdata[bdata["symbol"] == ticker]["q_datetime"].sample().item()
-        if qdate.month in [3,6,9,12]:
-            ticker_quarters[ticker] = 3
-        elif qdate.month in [2,5,8,11]:
-            ticker_quarters[ticker] = 2
-        elif qdate.month in [1,4,7,10]:
-            ticker_quarters[ticker] = 1
+        ticker_quarters[ticker] = choose_calendar_type(list(bdata[bdata["symbol"] == ticker]["q_datetime"]))
 
     ew_data["Month"] = ew_data["Time Periods"].apply(lambda x: datetime.datetime.strptime(x, "%b %Y"))
     ew_data["calendar_type"] = ew_data["Ticker"].apply(lambda x: ticker_quarters[x])
@@ -44,10 +67,17 @@ def create_dataset(ew_data):
     bdata["calendar_type"] = bdata["symbol"].apply(lambda x: ticker_quarters[x])
     bdata["quarter_end"] = bdata.apply(lambda x: get_quarter_end(x["q_datetime"], x["calendar_type"]), axis=1)
 
+    if log:
+        logging.info("Transforming Equity Watch data")
+    num_months = ew_data.groupby(["Ticker","quarter_end"])["Month"].nunique()
+    valid_quarter = num_months == 3
     ew_data = ew_data.groupby(["Ticker","quarter_end","Category"])["Dollars Adjusted"].sum().reset_index()
-    ew_data = ew_data.pivot(index=["Ticker","quarter_end"], columns="Category", values="Dollars Adjusted").reset_index()
-    ew_data = ew_data.rename(columns={"Ticker":"symbol"})
+    ew_data = ew_data.pivot(index=["Ticker","quarter_end"], columns="Category", values="Dollars Adjusted")
+    ew_data["valid_quarter"] = valid_quarter
+    ew_data = ew_data.reset_index().rename(columns={"Ticker":"symbol"})
     
+    if log:
+        logging.info("Mergin Equity Watch and Bloomberg data")
     all_data = ew_data.merge(bdata, on=["symbol","quarter_end"])
     all_data["total_dollars"] = all_data[[x for x in all_data.columns if x not in NON_SALES_COLS]].sum(axis=1)
     return all_data
@@ -86,36 +116,38 @@ def run_summary_cat_report(output_dir, df):
     writer.close()
 
 
-def most_corr_by_ticker_yy(data, measure, measure_est):
+def most_corr_by_ticker_yy(data, measure, measure_est, min_periods=4):
 
     yy_corr_df = pd.DataFrame()
     df = data.copy()
+    df = df[df["valid_quarter"]]
     for s in df["symbol"].unique():
-        
+        #print(s)
         sdf = df[df["symbol"] == s]
-        sdf = sdf.dropna(axis=1, thresh=len(sdf)-1)
-        if measure not in sdf.columns:
-            continue
+        
+        #if measure not in sdf.columns:
+            #continue
         use_cols = ["total_dollars",measure]
         if measure_est != None and measure_est in sdf.columns:
             use_cols.append(measure_est)
         share_percs = {}
         for x in sdf.columns:
             if x not in NON_SALES_COLS:
-                sdf[x] = sdf[x].apply(lambda x: 0 if pd.isnull(x) else x)
                 share_perc = (sdf[x] / sdf["total_dollars"]).mean()
                 share_percs[x] = share_perc
                 if share_perc > 0.01:
                     use_cols.append(x)
+        if use_cols in [["total_dollars",measure],["total_dollars",measure,measure_est]]:
+            continue 
+        sdf = sdf.set_index("quarter_end")
         for x in sdf.columns:
             if x not in NON_SALES_COLS or x in ["total_dollars",measure, measure_est]:
-                sdf[x] = sdf[x].pct_change(periods=4)
-        sdf = sdf.iloc[4:, :]
-        if len(sdf) < 2:
-            continue
-        #print(sdf[[measure, measure_est]])
-        cor = sdf[use_cols].corr()
+                sdf[x] = sdf[x].pct_change(freq=pd.DateOffset(years=1))
+        sdf = sdf.reset_index()
+        cor = sdf[use_cols].corr(min_periods=min_periods)
         cor = cor.sort_values(measure, ascending=False)
+        if cor.dropna().empty:
+            continue
         total_cor = cor.loc["total_dollars",measure]
         if measure_est in use_cols:
             est_cor = cor.loc[measure_est, measure]
@@ -123,6 +155,7 @@ def most_corr_by_ticker_yy(data, measure, measure_est):
             est_cor = None
         cor = cor.reset_index().rename(columns={"index":"category"})
         cor = cor[~cor["category"].isin(["total_dollars",measure, measure_est])].reset_index(drop=True)
+
         yy_corr_df = yy_corr_df.append({"symbol":s, "category":cor.loc[0, "category"], "category_corr":cor.loc[0, measure], "total_dollars_corr":total_cor, "share_of_total_dollars":share_percs[cor.loc[0,"category"]], "estimate_corr":est_cor}, ignore_index=True)
     return yy_corr_df.sort_values("category_corr", ascending=False)
 
